@@ -16,10 +16,13 @@ from pythonz.util import symlink, makedirs, Package, is_url, Link,\
     get_macosx_deployment_target, Version, is_python25, is_python24, is_python33
 from pythonz.define import PATH_BUILD, PATH_DISTS, PATH_PYTHONS, PATH_LOG, \
     PATH_PATCHES_ALL, PATH_PATCHES_OSX
-from pythonz.downloader import Downloader, DownloadError, validate_sha256
+from pythonz.downloader import Downloader, DownloadError, sha256
 from pythonz.log import logger
 
 
+class AlreadyInstalledError(Exception): pass
+
+
 class PythonInstaller(object):
     @staticmethod
     def get_installer(version, options):
@@ -72,6 +75,8 @@ class Installer(object):
         if os.path.isdir(self.install_dir):
             if options.reinstall:
                 shutil.rmtree(self.install_dir)
+            else:
+                raise AlreadyInstalledError("You have already installed `%s`" % self.pkg.name)
 
         self.options = options
         self.logfile = os.path.join(PATH_LOG, 'build.log')
@@ -87,22 +92,16 @@ class Installer(object):
         return self.supported_versions.get(self.pkg.version)
 
     def download(self):
-        if os.path.isfile(self.download_file):
+        if os.path.isfile(self.download_file) and sha256(self.download_file) == self.expected_sha256:
             logger.info("Use the previously fetched %s" % (self.download_file))
         else:
             base_url = Link(self.download_url).base_url
             logger.info("Downloading %s as %s" % (base_url, self.download_file))
             try:
-                Downloader.fetch(self.download_url, self.download_file)
+                Downloader.fetch(self.download_url, self.download_file, self.expected_sha256)
             except DownloadError:
-                unlink(self.download_file)
                 logger.error("Failed to download.\n%s" % (sys.exc_info()[1]))
-                sys.exit(1)
-            except:
-                unlink(self.download_file)
                 raise
-        if not validate_sha256(self.download_file, self.expected_sha256):
-            raise DownloadError("Corrupted download, the sha256 doesn't match")
 
     def install(self):
         raise NotImplementedError
@@ -160,9 +159,8 @@ class CPythonInstaller(Installer):
                         s.shell('ed - %s < %s' % (source, ed))
                 else:
                     s.shell("patch -p0 < %s" % patch)
-        except:
-            logger.error("Failed to patch `%s`.\n%s" % (self.build_dir, sys.exc_info()[1]))
-            sys.exit(1)
+        except Exception:
+            raise RuntimeError("Failed to patch `%s`.\n%s" % (self.build_dir, sys.exc_info()[1]))
 
     def _append_patch(self, patch_dir, patch_files):
         for patch in patch_files:
@@ -188,10 +186,6 @@ class CPythonInstaller(Installer):
             logger.error("Invalid content-type: `%s`" % self.content_type)
             return
 
-        if os.path.isdir(self.install_dir):
-            logger.info("You have already installed `%s`" % self.pkg.name)
-            return
-
         self.download_and_extract()
         logger.info("\nThis could take a while. You can run the following command on another shell to track the status:")
         logger.info("  tail -f %s\n" % self.logfile)
@@ -206,14 +200,13 @@ class CPythonInstaller(Installer):
             traceback.print_exc()
             rm_r(self.install_dir)
             logger.error("Failed to install %s. Check %s to see why." % (self.pkg.name, self.logfile))
-            sys.exit(1)
+            raise
         self.symlink()
         logger.info("\nInstalled %(pkgname)s successfully." % {"pkgname": self.pkg.name})
 
     def download_and_extract(self):
         self.download()
-        if not extract_downloadfile(self.content_type, self.download_file, self.build_dir):
-            sys.exit(1)
+        extract_downloadfile(self.content_type, self.download_file, self.build_dir)
 
     def _patch(self):
         version = Version(self.pkg.version)
@@ -430,10 +423,6 @@ class PyPyInstaller(Installer):
             logger.error("Invalid content-type: `%s`" % self.content_type)
             return
 
-        if os.path.isdir(self.install_dir):
-            logger.info("You have already installed `%s`" % self.pkg.name)
-            return
-
         self.download_and_extract()
         logger.info("Installing %s into %s" % (self.pkg.name, self.install_dir))
         shutil.copytree(self.build_dir, self.install_dir)
@@ -442,8 +431,7 @@ class PyPyInstaller(Installer):
 
     def download_and_extract(self):
         self.download()
-        if not extract_downloadfile(self.content_type, self.download_file, self.build_dir):
-            sys.exit(1)
+        extract_downloadfile(self.content_type, self.download_file, self.build_dir)
 
     def symlink(self):
         install_dir = os.path.realpath(self.install_dir)
@@ -503,10 +491,6 @@ class JythonInstaller(Installer):
             logger.error("Invalid content-type: `%s`" % self.content_type)
             return
 
-        if os.path.isdir(self.install_dir):
-            logger.info("You have already installed `%s`" % self.pkg.name)
-            return
-
         self.download()
         logger.info("\nThis could take a while. You can run the following command on another shell to track the status:")
         logger.info("  tail -f %s\n" % self.logfile)
